/***************************************************************************
// Written by J. Keller, julien.keller@univ-amu.fr (2014) //                                   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "in-output.h"

////////////////////////////////
//ResultsStore Implementations//
////////////////////////////////

void ResultsStore::addRow(double z1S, double z2S, double x1S, double x2S, double cutOffS, double volumeFormS, double svS)
{
  z1.push_back(z1S);
  z2.push_back(z2S);
  x1.push_back(x1S);
  x2.push_back(x2S);
  cutOff.push_back(cutOffS);
  volumeForm.push_back(volumeFormS);
  sv.push_back(svS);
}

void ResultsStore::clear()
{
  z1.clear();
  z2.clear();
  x1.clear();
  x2.clear();
  cutOff.clear();
  volumeForm.clear();
  sv.clear();
}

void ResultsStore::print() const
{
  cout << "             z1             z2             x1             x2           volF             sv         cutOff"<< endl;
  for(int i = 0; i < z1.size(); i++)
  {
    cout.width(15);
    cout << z1[i];
    cout.width(15);
    cout << z2[i];
    cout.width(15);
    cout << x1[i];
    cout.width(15);
    cout << x2[i];
    cout.width(15);
    cout << volumeForm[i];
    cout.width(15);
    cout << sv[i];
    cout.width(15);
    cout << cutOff[i];
    cout << endl;
  }
}



/////////////////////////////////
//Output to file Implementation//
/////////////////////////////////

void write_results_in_file(MMatrix<double>* p_an,int method)
{
    const MMatrix<double> & an = *p_an;
    bool possible=0;
    int numb=0;
    std::string name_file;  //name of file for output
    while(possible==0)
    {
      std::ostringstream s2;
      s2 << "res" << numb <<".txt";
      name_file=s2.str();
      ifstream file(name_file.c_str());
      if(file==NULL)
      {
        possible=1;
        ofstream file(name_file.c_str());
        cout << "\n Writing data in file " << name_file <<endl;
        file << "Metric obtained by";
        if(method==0){
          file << " Balanced method"<<endl;
        }
        if(method==1){
          file << " Ricci method"<<endl;
        }
        file<< an << endl;
        an.writeToStream(file);
        file.close();
      }
      else numb++;
    }
  
  //ofstream file(name_file.c_str(),ofstream::app);
}


void print_time(char* a,struct timeval *Tpf,struct timeval *Tps){
    printf("%s %ld sec",a,
           (Tpf->tv_usec-Tps->tv_usec<0)?(Tpf->tv_sec-Tps->tv_sec-1):(Tpf->tv_sec-Tps->tv_sec));
  printf(" %ld msec", (Tpf->tv_usec-Tps->tv_usec<0)?(1000000+(Tpf->tv_usec-Tps->tv_usec))/1000:
          ((Tpf->tv_usec-Tps->tv_usec))/1000);
  printf("\n");
}



