/***************************************************************************
 *   Copyright (C) 2007 by Julien Keller   *
 *   j.keller@imperial.ac.uk   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __M_MATRIX2_H__
#define __M_MATRIX2_H__ 
#include <complex>
#define Complex  std::complex<double>
#include "MMatrix.h"  //Contains library to handle quickly matrices in memory
#include <iostream>


template <typename T>
class VVector : public MMatrix<T>
{
    public:
        T& operator[](unsigned int index) {return this->getData()[0][index];}
        const T& operator[](unsigned int index) const {return this->getData()[0][index];}
  //  VVector(unsigned int height=0,const T& fill=T())
   // {this.fillWith(height,1,0,fill);}
}; 


/* See m_matrix.cpp file for details on procedures */

void show(MMatrix<double>);
void show(MMatrix<int>);
void show(MMatrix<Complex>);
void show(MMatrix<double>,char*);

//MMatrix<double>& invertNonZeroElements(MMatrix<double>& M); //inverts each non-zero element of M and replaces it in M. Zero elements are left 0. Another reference to M is returned for convenience.
//MMatrix<double> inverse_elements(MMatrix<double>);
//MMatrix<double> transpose(MMatrix<double>);
//MMatrix<double> transpose2(MMatrix<double>);	
//double  scalar_product(MMatrix<double>,MMatrix<double>);


/* These procedures can always be improved */
//MMatrix<double> multiply(MMatrix<double>,MMatrix<double>);
//double spectral_radius(MMatrix<double>);

MMatrix<double> real(MMatrix<Complex> A);
MMatrix<Complex> conj(MMatrix<Complex> A);
MMatrix<Complex> MMatrixDoubleToComplex(MMatrix<double> A);


/* Simple procedure to multiply 2 matrices probably less efficient than BLAS*/
/*
template<class T> MMatrix<T> multiply(const MMatrix<T> A,const MMatrix<T> B)
{
    MMatrix<double> Temp(A.getHeight(),B.getWidth());

    int i,j,k;
    for(i=0;i<A.getHeight();i++)
    {
        for(j=0;j<B.getWidth();j++)
        {
            for(k=0;k<A.getWidth();k++)
            {
                Temp[i][j]+=(A[i][k]*B[k][j]);
            }
        }
    }
    return Temp;
}



template<class T> T determinant(const MMatrix<T> A)
{
    if(A.getHeight()!=A.getWidth()){
        return 0;
    }
    if (A.getHeight()==1){
        return A[0][0];
    }
    if (A.getHeight()==2){
        return A[0][0]*A[1][1]-A[1][0]*A[0][1];
    }
    if (A.getHeight()==3){
        return (A[0][0]*A[1][1]*A[2][2]+A[0][1]*A[1][2]*A[2][0]+A[1][0]*A[2][1]*A[0][2]
                -A[2][0]*A[1][1]*A[0][2]-A[0][1]*A[1][0]*A[2][2]-A[0][0]*A[2][1]*A[1][2]);
    }
    if (A.getHeight()==4){
        return(A[0][0]*A[1][1]*A[2][2]*A[3][3]
                -A[0][0]*A[1][1]*A[2][3]*A[3][2]
                -A[0][0]*A[2][1]*A[1][2]*A[3][3]
                +A[0][0]*A[2][1]*A[1][3]*A[3][2]
                +A[0][0]*A[3][1]*A[1][2]*A[2][3]
                -A[0][0]*A[3][1]*A[1][3]*A[2][2]
                -A[1][0]*A[0][1]*A[2][2]*A[3][3]
                +A[1][0]*A[0][1]*A[2][3]*A[3][2]
                +A[1][0]*A[2][1]*A[0][2]*A[3][3]
                -A[1][0]*A[2][1]*A[0][3]*A[3][2]
                -A[1][0]*A[3][1]*A[0][2]*A[2][3]
                +A[1][0]*A[3][1]*A[0][3]*A[2][2]
                +A[2][0]*A[0][1]*A[1][2]*A[3][3]
                -A[2][0]*A[0][1]*A[1][3]*A[3][2]
                -A[2][0]*A[1][1]*A[0][2]*A[3][3]
                +A[2][0]*A[1][1]*A[0][3]*A[3][2]
                +A[2][0]*A[3][1]*A[0][2]*A[1][3]
                -A[2][0]*A[3][1]*A[0][3]*A[1][2]
                -A[3][0]*A[0][1]*A[1][2]*A[2][3]
                +A[3][0]*A[0][1]*A[1][3]*A[2][2]
                +A[3][0]*A[1][1]*A[0][2]*A[2][3]
                -A[3][0]*A[1][1]*A[0][3]*A[2][2]
                -A[3][0]*A[2][1]*A[0][2]*A[1][3]
                +A[3][0]*A[2][1]*A[0][3]*A[1][2]);
    }
    if (A.getHeight()==5){
        return(
                A[0][0]*A[1][1]*A[2][2]*A[3][3]*A[4][4]-A[0][0]*A[1][1]*A[2][2]*A[3][4]*A[4][3]-A[0][0]*A[1][1]*A[3][2]*A[2][3]*A[4][4]+A[0][0]*A[1][1]*A[3][2]*A[2][4]*A[4][3]+A[0][0]*A[1][1]*A[4][2]*A[2][3]*A[3][4]-A[0][0]*A[1][1]*A[4][2]*A[2][4]*A[3][3]-A[0][0]*A[2][1]*A[1][2]*A[3][3]*A[4][4]+A[0][0]*A[2][1]*A[1][2]*A[3][4]*A[4][3]+A[0][0]*A[2][1]*A[3][2]*A[1][3]*A[4][4]-A[0][0]*A[2][1]*A[3][2]*A[1][4]*A[4][3]-A[0][0]*A[2][1]*A[4][2]*A[1][3]*A[3][4]+A[0][0]*A[2][1]*A[4][2]*A[1][4]*A[3][3]+A[0][0]*A[3][1]*A[1][2]*A[2][3]*A[4][4]-A[0][0]*A[3][1]*A[1][2]*A[2][4]*A[4][3]-A[0][0]*A[3][1]*A[2][2]*A[1][3]*A[4][4]+A[0][0]*A[3][1]*A[2][2]*A[1][4]*A[4][3]+A[0][0]*A[3][1]*A[4][2]*A[1][3]*A[2][4]-A[0][0]*A[3][1]*A[4][2]*A[1][4]*A[2][3]-A[0][0]*A[4][1]*A[1][2]*A[2][3]*A[3][4]+A[0][0]*A[4][1]*A[1][2]*A[2][4]*A[3][3]+A[0][0]*A[4][1]*A[2][2]*A[1][3]*A[3][4]-A[0][0]*A[4][1]*A[2][2]*A[1][4]*A[3][3]-A[0][0]*A[4][1]*A[3][2]*A[1][3]*A[2][4]+A[0][0]*A[4][1]*A[3][2]*A[1][4]*A[2][3]-A[1][0]*A[0][1]*A[2][2]*A[3][3]*A[4][4]+A[1][0]*A[0][1]*A[2][2]*A[3][4]*A[4][3]+A[1][0]*A[0][1]*A[3][2]*A[2][3]*A[4][4]-A[1][0]*A[0][1]*A[3][2]*A[2][4]*A[4][3]-A[1][0]*A[0][1]*A[4][2]*A[2][3]*A[3][4]+A[1][0]*A[0][1]*A[4][2]*A[2][4]*A[3][3]+A[1][0]*A[2][1]*A[0][2]*A[3][3]*A[4][4]-A[1][0]*A[2][1]*A[0][2]*A[3][4]*A[4][3]-A[1][0]*A[2][1]*A[3][2]*A[0][3]*A[4][4]+A[1][0]*A[2][1]*A[3][2]*A[0][4]*A[4][3]+A[1][0]*A[2][1]*A[4][2]*A[0][3]*A[3][4]-A[1][0]*A[2][1]*A[4][2]*A[0][4]*A[3][3]-A[1][0]*A[3][1]*A[0][2]*A[2][3]*A[4][4]+A[1][0]*A[3][1]*A[0][2]*A[2][4]*A[4][3]+A[1][0]*A[3][1]*A[2][2]*A[0][3]*A[4][4]-A[1][0]*A[3][1]*A[2][2]*A[0][4]*A[4][3]-A[1][0]*A[3][1]*A[4][2]*A[0][3]*A[2][4]+A[1][0]*A[3][1]*A[4][2]*A[0][4]*A[2][3]+A[1][0]*A[4][1]*A[0][2]*A[2][3]*A[3][4]-A[1][0]*A[4][1]*A[0][2]*A[2][4]*A[3][3]-A[1][0]*A[4][1]*A[2][2]*A[0][3]*A[3][4]+A[1][0]*A[4][1]*A[2][2]*A[0][4]*A[3][3]+A[1][0]*A[4][1]*A[3][2]*A[0][3]*A[2][4]-A[1][0]*A[4][1]*A[3][2]*A[0][4]*A[2][3]+A[2][0]*A[0][1]*A[1][2]*A[3][3]*A[4][4]-A[2][0]*A[0][1]*A[1][2]*A[3][4]*A[4][3]-A[2][0]*A[0][1]*A[3][2]*A[1][3]*A[4][4]+A[2][0]*A[0][1]*A[3][2]*A[1][4]*A[4][3]+A[2][0]*A[0][1]*A[4][2]*A[1][3]*A[3][4]-A[2][0]*A[0][1]*A[4][2]*A[1][4]*A[3][3]-A[2][0]*A[1][1]*A[0][2]*A[3][3]*A[4][4]+A[2][0]*A[1][1]*A[0][2]*A[3][4]*A[4][3]+A[2][0]*A[1][1]*A[3][2]*A[0][3]*A[4][4]-A[2][0]*A[1][1]*A[3][2]*A[0][4]*A[4][3]-A[2][0]*A[1][1]*A[4][2]*A[0][3]*A[3][4]+A[2][0]*A[1][1]*A[4][2]*A[0][4]*A[3][3]+A[2][0]*A[3][1]*A[0][2]*A[1][3]*A[4][4]-A[2][0]*A[3][1]*A[0][2]*A[1][4]*A[4][3]-A[2][0]*A[3][1]*A[1][2]*A[0][3]*A[4][4]+A[2][0]*A[3][1]*A[1][2]*A[0][4]*A[4][3]+A[2][0]*A[3][1]*A[4][2]*A[0][3]*A[1][4]-A[2][0]*A[3][1]*A[4][2]*A[0][4]*A[1][3]-A[2][0]*A[4][1]*A[0][2]*A[1][3]*A[3][4]+A[2][0]*A[4][1]*A[0][2]*A[1][4]*A[3][3]+A[2][0]*A[4][1]*A[1][2]*A[0][3]*A[3][4]-A[2][0]*A[4][1]*A[1][2]*A[0][4]*A[3][3]-A[2][0]*A[4][1]*A[3][2]*A[0][3]*A[1][4]+A[2][0]*A[4][1]*A[3][2]*A[0][4]*A[1][3]-A[3][0]*A[0][1]*A[1][2]*A[2][3]*A[4][4]+A[3][0]*A[0][1]*A[1][2]*A[2][4]*A[4][3]+A[3][0]*A[0][1]*A[2][2]*A[1][3]*A[4][4]-A[3][0]*A[0][1]*A[2][2]*A[1][4]*A[4][3]-A[3][0]*A[0][1]*A[4][2]*A[1][3]*A[2][4]+A[3][0]*A[0][1]*A[4][2]*A[1][4]*A[2][3]+A[3][0]*A[1][1]*A[0][2]*A[2][3]*A[4][4]-A[3][0]*A[1][1]*A[0][2]*A[2][4]*A[4][3]-A[3][0]*A[1][1]*A[2][2]*A[0][3]*A[4][4]+A[3][0]*A[1][1]*A[2][2]*A[0][4]*A[4][3]+A[3][0]*A[1][1]*A[4][2]*A[0][3]*A[2][4]-A[3][0]*A[1][1]*A[4][2]*A[0][4]*A[2][3]-A[3][0]*A[2][1]*A[0][2]*A[1][3]*A[4][4]+A[3][0]*A[2][1]*A[0][2]*A[1][4]*A[4][3]+A[3][0]*A[2][1]*A[1][2]*A[0][3]*A[4][4]-A[3][0]*A[2][1]*A[1][2]*A[0][4]*A[4][3]-A[3][0]*A[2][1]*A[4][2]*A[0][3]*A[1][4]+A[3][0]*A[2][1]*A[4][2]*A[0][4]*A[1][3]+A[3][0]*A[4][1]*A[0][2]*A[1][3]*A[2][4]-A[3][0]*A[4][1]*A[0][2]*A[1][4]*A[2][3]-A[3][0]*A[4][1]*A[1][2]*A[0][3]*A[2][4]+A[3][0]*A[4][1]*A[1][2]*A[0][4]*A[2][3]+A[3][0]*A[4][1]*A[2][2]*A[0][3]*A[1][4]-A[3][0]*A[4][1]*A[2][2]*A[0][4]*A[1][3]+A[4][0]*A[0][1]*A[1][2]*A[2][3]*A[3][4]-A[4][0]*A[0][1]*A[1][2]*A[2][4]*A[3][3]-A[4][0]*A[0][1]*A[2][2]*A[1][3]*A[3][4]+A[4][0]*A[0][1]*A[2][2]*A[1][4]*A[3][3]+A[4][0]*A[0][1]*A[3][2]*A[1][3]*A[2][4]-A[4][0]*A[0][1]*A[3][2]*A[1][4]*A[2][3]-A[4][0]*A[1][1]*A[0][2]*A[2][3]*A[3][4]+A[4][0]*A[1][1]*A[0][2]*A[2][4]*A[3][3]+A[4][0]*A[1][1]*A[2][2]*A[0][3]*A[3][4]-A[4][0]*A[1][1]*A[2][2]*A[0][4]*A[3][3]-A[4][0]*A[1][1]*A[3][2]*A[0][3]*A[2][4]+A[4][0]*A[1][1]*A[3][2]*A[0][4]*A[2][3]+A[4][0]*A[2][1]*A[0][2]*A[1][3]*A[3][4]-A[4][0]*A[2][1]*A[0][2]*A[1][4]*A[3][3]-A[4][0]*A[2][1]*A[1][2]*A[0][3]*A[3][4]+A[4][0]*A[2][1]*A[1][2]*A[0][4]*A[3][3]+A[4][0]*A[2][1]*A[3][2]*A[0][3]*A[1][4]-A[4][0]*A[2][1]*A[3][2]*A[0][4]*A[1][3]-A[4][0]*A[3][1]*A[0][2]*A[1][3]*A[2][4]+A[4][0]*A[3][1]*A[0][2]*A[1][4]*A[2][3]+A[4][0]*A[3][1]*A[1][2]*A[0][3]*A[2][4]-A[4][0]*A[3][1]*A[1][2]*A[0][4]*A[2][3]-A[4][0]*A[3][1]*A[2][2]*A[0][3]*A[1][4]+A[4][0]*A[3][1]*A[2][2]*A[0][4]*A[1][3]);     
    }
}

*/
#endif
